import java.util.*;

public class MapDemo
{
   private static final String[] input = {
      "Aida,Verdi",
      "Billy Budd,Britten",
      "Das Rheingold,Wagner",
      "Die Walkre,Wagner",
      "Die Zauberflte,Mozart",
      "Ernani,Verdi",
      "Gloriana,Britten",
      "La Bohme,Puccini",
      "La Fanciulla del West,Puccini",
      "La Forza del Destino,Verdi",
      "La Traviata,Verdi",
      "Le Nozze di Figaro,Mozart",
      "Lohengrin,Wagner",
      "Macbeth,Verdi",
      "Madama Butterfly,Puccini",
      "Manon Lescaut,Puccini",
      "Peter Grimes,Britten",
      "Rigoletto,Verdi",
      "Tosca,Puccini",
      "Turandot,Puccini",
   };

   public void run()
   {
      // Create a map that links a composer name
      // to an information string that lists the
      // full name, nationality, year of birth,
      // and year of death.

      Map infoMap = new HashMap();

      infoMap.put("Verdi",
               "Guiseppe Verdi (Italian, 1813-1901)");
      infoMap.put("Puccini",
               "Giacomo Puccini (Italian, 1858-1924)");
      infoMap.put("Mozart",
               "Wolfgang Amadeus Mozart (Austrian, 1756-1791)");
      infoMap.put("Wagner",
               "Richard Wagner (German, 1813-1883)");
      infoMap.put("Britten",
               "Benjamin Britten (British, 1913-1976)");

      // Create a map that will associate a composer name
      // with a set of operas by that composer

      Map operasByComposer = new TreeMap();
      for (int i = 0; i < input.length; i++) {

         // Get each line of input and split it
         // into the title of the opera and the
         // name of the composer

         String line = input[i];
         int p = line.indexOf(",");
         String opera = line.substring(0, p);
         String composer = line.substring(p+1);

         // If this is the first time we have encountered
         // this composer, map the composer name to an
         // empty set

         if (!operasByComposer.containsKey(composer)) {
            operasByComposer.put(composer, new TreeSet());
         }

         // Retreive the set of operas by this composer
         // and add the new one

         Set operas = (Set) operasByComposer.get(composer);
         operas.add(opera);
      }

      // Retrieve the set of composers 
      // and report the number found

      Set composers = operasByComposer.keySet();
      int n = composers.size();
      System.out.println
         ("There are " + n + " composers in the list:");

      // Iterate through the composer set

      for (Iterator it = composers.iterator(); it.hasNext(); ) {
         System.out.println();

         // Get the composer short name and look up
         // the full name and information string

         String composer = (String) it.next();
         String info = (String) infoMap.get(composer);
         System.out.println(info); 

         // Retrieve the set of operas by this composer

         Set operas = (Set) operasByComposer.get(composer);

         // Iterate through the set and print each element

         for (Iterator it2 = operas.iterator(); it2.hasNext(); ) {
            String opera = (String) it2.next();
            System.out.println("   " + opera);
         }
      }
   }

   public static void main(String[] args)
   {
      new MapDemo().run();
   }
}
