// Demonstrates the effect of concurrent threads accessing 
// a shared resource without synchronization.
public class AccountDemo  implements Runnable
{
	private Account myAccount = new Account();
	
	public static void main ( String s[] )
	{
		AccountDemo a1 = new AccountDemo();
		
		// Start two threads using the same target.
		Thread t1 = new Thread( a1 ) ;
		Thread t2 = new Thread( a1 ) ;
		
		System.out.println( "Starting thread 1 ");
		t1.start();
		
		System.out.println( "Starting thread 2 ");
		t2.start();
	}
	
	public void run()
	{
		// Save the balance for cross-checking after transactions.
		int beginBalance = myAccount.getBalance();
		for ( long i=0 ; i < 500000 ; i++ )
		{
			// These two operations typify a transaction. 
			myAccount.deposit( 200 );
			Thread.yield();
			myAccount.withdraw( 200 );
		
			// Inspect if the result of the transaction is consistent.
			if ( myAccount.getBalance() != beginBalance )
				System.out.println( "Iteration # " + i + 
									 " Begin balance = " + beginBalance + 
									 " End balance = " + myAccount.getBalance());
		}
	}
}