import java.util.Vector; 
 
class WordList 
{
    static final int MAX_SIZE= 5; 

	// Sample list of words for testing.
	static final String dictionary []  = 
				{
					"Monday", "Tuesday","Wednesday", "cat", "rat","mat",
					"boy","Java","World", "certification", "wonderful", 
				  	"broom", "brick", "house", "mouse", "Thursday", 
				  	"August", "December", "Fall", "winter", "morning"
				 };
	
    // Shared data buffer.
    private Vector NewWords = new Vector(); 
    
    private boolean listFull() { return ( NewWords.size() == MAX_SIZE - 1) ; }

    private boolean listEmpty() { return NewWords.isEmpty(); }


    synchronized public void produce()
    { 
       	while ( listFull() )
       	{
            try
            {
            	wait(); 
        		System.out.println("List full. Producer waiting for consumers to consume....");
            }
            catch (InterruptedException  e ) {}
       	}
       	
       	// Generate a random number and pick a word from the dictionary.
       	int randomIndex = (int) (Math.random() * dictionary.length);
    	System.out.println( "Produced : " + dictionary[randomIndex]);

		//add the new word to the list ...
        NewWords.addElement( new String(dictionary[randomIndex]) ); 
        
        //.. and notify the conumser
        notify(); 
    } 
	
    // Called by Consumer 
    synchronized public void consume()  
    { 
        while ( listEmpty() ) 
        {
            try
            {
        		System.out.println("List empty. Consumer waiting for producer for more data....");
            	wait(); 
            }
            catch (InterruptedException  e ) {}
        }
        
        // Get the last word from the word list.
        String message = (String)NewWords.lastElement(); 
        NewWords.removeElement( message ); 
    	System.out.println( "Consumed : " + message);
        
        //.. and notify the producer
        notify(); 
    } 
}


final class Producer extends Thread
{ 
	private WordList wordList ; 

	public Producer( WordList w )
	{
		this.wordList = w ;
		this.start();
	}
	
    public void run() 
    { 
        try 
        { 
            while ( true ) 
            { 
                wordList.produce(); 
                // sleep for a random interval of time.
                sleep( (int) (Math.random() * 500 )); 
            } 
        }  
        catch( InterruptedException e ) { } 
    } 
} 

 
final class Consumer extends Thread
{
	private WordList wordList ; 
   
	public Consumer( WordList w )
	{
		this.wordList = w ;
		this.setDaemon(true);
		this.start();
	}
  
    public void run() 
    { 
        try { 
            while ( true ) 
            { 
				wordList.consume();
                // sleep for a random interval of time.
                sleep( (int) (Math.random() * 500 )); 
            } 
        }  
        catch( InterruptedException e ) { } 
    } 
} 


public class ProducerConsumerDemo
{
    public static void main(String args[]) 
    { 
		WordList w = new WordList();
        Producer producer = new Producer( w ); 
        Consumer consumer = new Consumer( w );
    } 
}