import java.io.*;

/**
* An example of using a RandomAccessFile to read
* metadadta from a file in dBase format.
*/
public class RandomAccessExample
{
   public RandomAccessExample(String fileName)
      throws IOException
   {
      // Open the dBase file for random access

      RandomAccessFile raf = new RandomAccessFile(fileName, "r");

      // Field descriptors are 32-byte structures,
      // beginning at offset 32 into the header.

      final int DESCRIPTOR_OFFSET = 32;
      final int DESCRIPTOR_LENGTH = 32;

      // Read and print each field descriptor

      for (int fieldNumber = 0;; fieldNumber++) {

         long pos = DESCRIPTOR_OFFSET
            + fieldNumber * DESCRIPTOR_LENGTH;

         raf.seek(pos);

         // A 0x0D byte signals the end of field
         // descriptors

         byte b = raf.readByte();
         if (b == 0x0D)
            break;

         // If not end of descriptors, this is the
         // first byte of a field name.  Read the
         // rest of the name.

         byte[] buffer = new byte[11];
         buffer[0] = b;
         raf.readFully(buffer, 1, 10);

         // The name is exactly 11 bytes, in
         // ASCII with trailing zeros

         int z = 0;
         while (buffer[z] != 0)
            z++;
         String name = new String(buffer, 0, z);

         // The next byte is the field type:
         // C, N, L, D, or M

         char type = (char) raf.read();

         // Skip 4 bytes of the field address

         raf.skipBytes(4);

         // Then read the one-byte field length

         int length = raf.readByte();

         // Print results

         StringBuffer sb = new StringBuffer();
         sb.append("Field ");
         sb.append(fieldNumber + 1);
         sb.append(": name=");
         sb.append(name);
         sb.append(", type=");
         sb.append(type);
         sb.append(", length=");
         sb.append(length);
         System.out.println(sb);
      }
      raf.close();
   }

   /**
   * Mainline
   */
   public static void main(String[] args)
      throws IOException
   {
      String fileName = args[0];
      new RandomAccessExample(fileName);
   }
}
